﻿using System;
using System.Text;

namespace CSCUSTOM
{
    public static class NumberToString
    {
        private static readonly string[] sFrac20Male = new[]
                                                           {
                                                                   "", "один", "два", "три", "четыре", "пять", "шесть", "семь", "восемь", "девять", "десять", "одиннадцать", "двенадцать", "тринадцать",
                                                                   "четырнадцать", "пятнадцать", "шестнадцать", "семнадцать", "восемнадцать", "девятнадцать"
                                                           };

        private static readonly string[] sFrac20Female = new[]
                                                             {
                                                                     "", "одна", "две", "три", "четыре", "пять", "шесть", "семь", "восемь", "девять", "десять", "одиннадцать", "двенадцать",
                                                                     "тринадцать",
                                                                     "четырнадцать", "пятнадцать", "шестнадцать", "семнадцать", "восемнадцать", "девятнадцать"
                                                             };

        private static readonly string[] sHundreds = new[] { "", "сто", "двести", "триста", "четыреста", "пятьсот", "шестьсот", "семьсот", "восемьсот", "девятьсот" };
        private static readonly string[] sTens = new[] { "", "десять", "двадцать", "тридцать", "сорок", "пятьдесят", "шестьдесят", "семьдесят", "восемьдесят", "девяносто" };

        public static string Convert( decimal dValue, bool bIsMale )
        {
            if ( dValue >= 1000000000000000 )
            {
                throw new ArgumentOutOfRangeException( "dValue" );
            }

            var res = new StringBuilder();

            if ( dValue < 0 )
            {
                res.Append( "минус" );
                dValue = -dValue;
            }

            if ( dValue == 0 )
            {
                return "ноль";
            }

            dValue = dValue
                    .AppendPeriod( 1000000000000, res, "триллион", "триллиона", "триллионов", true )
                    .AppendPeriod( 1000000000, res, "миллиард", "миллиарда", "миллиардов", true )
                    .AppendPeriod( 1000000, res, "миллион", "миллиона", "миллионов", true )
                    .AppendPeriod( 1000, res, "тысяча", "тысячи", "тысяч", false );

            var iHundreds = ( int ) ( dValue / 100 );
            if ( iHundreds != 0 )
            {
                res.AppendWithSpace( sHundreds[ iHundreds ] );
            }

            var iLess100 = ( int ) ( dValue % 100 );
            string[] sFrac20 = bIsMale ? sFrac20Male : sFrac20Female;
            if ( iLess100 < 20 )
            {
                res.AppendWithSpace( sFrac20[ iLess100 ] );
            }
            else
            {
                int iTens = iLess100 / 10;
                res.AppendWithSpace( sTens[ iTens ] );

                int iLess10 = iLess100 % 10;
                if ( iLess10 != 0 )
                {
                    res.Append( " " + sFrac20[ iLess10 ] );
                }
            }

            return res.ToString();
        }

        private static string Convert( this decimal dValue, bool bIsMale, string sDeclension1, string sDeclension2, string sDeclension5 )
        {
            return Convert( dValue, bIsMale ) + " " + ( ( int ) ( dValue % 10 ) ).GetDeclension( sDeclension1, sDeclension2, sDeclension5 );
        }

        private static decimal AppendPeriod( this decimal dValue, long lPower, StringBuilder stringBuilder, string sDeclension1, string sDeclension2, string sDeclension5, bool bIsMale )
        {
            var iThousands = ( int ) ( dValue / lPower );
            if ( iThousands > 0 )
            {
                stringBuilder.AppendWithSpace( ( ( decimal ) iThousands ).Convert( bIsMale, sDeclension1, sDeclension2, sDeclension5 ) );
                return dValue % lPower;
            }
            return dValue;
        }

        private static void AppendWithSpace( this StringBuilder stringBuilder, string sAppend )
        {
            if ( stringBuilder.Length > 0 )
            {
                stringBuilder.Append( " " );
            }
            stringBuilder.Append( sAppend );
        }

        private static string GetDeclension( this int iValue, string sDeclension1, string sDeclension2, string sDeclension5 )
        {
            int t = ( iValue % 100 > 20 ) ? iValue % 10 : iValue % 20;

            switch ( t )
            {
                case 1:
                    return sDeclension1;
                case 2:
                case 3:
                case 4:
                    return sDeclension2;
                default:
                    return sDeclension5;
            }
        }
    }
}